@@ -33,7 +33,9 @@ pub fn init(_: TokenStream, function: TokenStream) -> TokenStream {
 
             #[no_mangle]
             unsafe extern "C" fn init_plugin(plugin: *mut ::panda::PluginHandle) {
-                for cb in ::panda::inventory::iter::<::panda::Callback> {
+                ::panda::set_plugin_ref(plugin);
+
+                for cb in ::panda::inventory::iter::<::panda::InternalCallback> {
                     ::panda::sys::panda_register_callback(plugin as _, cb.cb_type, ::core::mem::transmute(cb.fn_pointer));
                 }
 
@@ -73,6 +75,67 @@ pub fn uninit(_: TokenStream, function: TokenStream) -> TokenStream {
     .into()
 }
 
+/// An attribute to declare a function for hooking using the PANDA 'hooks' plugin,
+/// enabling the ability to add callbacks for when a specifc instruction is hit,
+/// with control over the address space, kernel mode, and callback type to use.
+///
+/// ## Example
+///
+/// ```
+/// use panda::plugins::proc_start_linux::AuxvValues;
+/// use panda::plugins::hooks::Hook;
+/// use panda::prelude::*;
+///
+/// #[panda::hook]
+/// fn entry_hook(_: &mut CPUState, _: &mut TranslationBlock, _: u8, hook: &mut Hook) {
+///     println!("\n\nHit entry hook!\n");
+///
+///     // only run hook once
+///     hook.enabled = false;
+/// }
+///
+/// #[panda::on_rec_auxv]
+/// fn on_proc_start(_: &mut CPUState, _: &mut TranslationBlock, auxv: &AuxvValues) {
+///     // when a process starts, hook the entrypoint
+///     entry_hook::hook()
+///         .after_block_exec()
+///         .at_addr(auxv.entry)
+/// }
+///
+/// Panda::new()
+///     .generic("x86_64")
+///     .replay("test")
+///     .run();
+/// ```
+///
+/// ## Supported Callback Types
+///
+/// ### Standard callbacks
+///
+/// These callbacks take the form of:
+///
+/// ```
+/// #[panda::hook]
+/// fn my_callback(cpu: &mut CPUState, tb: &mut TranslationBlock, hook: &mut Hook);
+/// ```
+///
+/// |         Callback        | Info |
+/// |:------------------------|------|
+/// | `before_tcg_codegen`    | Callback at the start of the tcg IR being generated |
+/// | `after_block_translate` | Callback after the block the hooked instruction is in gets translated |
+/// | `before_block_exec`     | Callback before the block the given instruction is in gets run |
+/// | `start_block_exec`      | Callback at the first instruction in the block the instruction is in |
+/// | `end_block_exec`        | Callback after the last instruction in the block the hooked instruction is in |
+///
+/// ### Other Callbacks
+///
+/// These callbacks each have their own unique required function signature.
+///
+/// |          Callback        | Required Signature | Info |
+/// |:-------------------------|--------------------|------|
+/// | `before_block_translate` | `fn(cpu: &mut CPUState, pc: target_ptr_t, hook: &mut Hook)` | Callback that runs before the block the hooked instruction is translated to tcg |
+/// | `after_block_exec`       | `fn(cpu: &mut CPUState, tb: &mut TranslationBlock, exitCode: u8, hook: &mut Hook)` | Callback that runs after the given block is executed |
+/// | `before_block_exec_invalidate_opt` | `fn(env: &mut CPUState, tb: &mut TranslationBlock, hook: &mut Hook) -> bool` | Callback on translate to provide the option to invalidate the block the hooked instruction is generated in |
 #[proc_macro_attribute]
 pub fn hook(_: TokenStream, func: TokenStream) -> TokenStream {
     let mut function = syn::parse_macro_input!(func as syn::ItemFn);
@@ -255,7 +318,7 @@ fn get_cfg_attrs(func: &syn::ItemFn) -> Vec<syn::Attribute> {
 macro_rules! define_callback_attributes {
     ($(
         $($doc:literal)*
-        ($attr_name:ident, $const_name:ident, ($($arg:ty),*) $(-> $ret:ty)?)
+        ($attr_name:ident, $const_name:ident, ($($arg_name:ident : $arg:ty),*) $(-> $ret:ty)?)
     ),*) => {
         $(
             doc_comment::doc_comment!{
@@ -291,7 +354,7 @@ macro_rules! define_callback_attributes {
 
                         ::panda::inventory::submit! {
                             #![crate = ::panda]
-                            ::panda::Callback::new(
+                            ::panda::InternalCallback::new(
                                 ::panda::sys::$const_name,
                                 #func as *const ()
                             )
@@ -335,6 +398,62 @@ macro_rules! define_callback_attributes {
                 }
             }
         )*
+
+        #[proc_macro]
+        pub fn define_closure_callbacks(_: TokenStream) -> TokenStream {
+            quote!(
+                impl Callback {
+                    $(
+                        /// Installs the given callback, assigning it to this `Callback`'s
+                        /// slot. Any callbacks previously stored in that slot will be
+                        /// freed.
+                        ///
+                        $(
+                            #[doc = $doc]
+                        )*
+                        pub fn $attr_name<F>(self, callback: F)
+                            where F: FnMut($($arg),*) $(-> $ret)? + 'static
+                        {
+                            unsafe extern "C" fn trampoline(context: *mut c_void, $($arg_name: $arg),*) $(-> $ret)? {
+                                let closure: &mut &mut (
+                                    dyn FnMut($($arg),*) $(-> $ret)?
+                                ) = unsafe { std::mem::transmute(
+                                    context as *mut *mut c_void
+                                )};
+                                closure($($arg_name),*)
+                            }
+
+                            unsafe fn drop_fn(this: *mut *mut c_void) {
+                                let _: Box<Box<dyn FnMut($($arg),*) $(-> $ret)?>> = unsafe {
+                                    std::mem::transmute(this)
+                                };
+                            }
+
+                            let closure_ref: *mut *mut c_void = unsafe {
+                                let x: Box<Box<
+                                    dyn FnMut($($arg),*) $(-> $ret)?
+                                >> = Box::new(
+                                    Box::new(callback) as Box<_>
+                                );
+
+                                std::mem::transmute(x)
+                            };
+
+                            install_closure_callback(self.0, ClosureCallback {
+                                closure_ref,
+                                drop_fn,
+                                trampoline: sys::panda_cb_with_context {
+                                    $attr_name: Some(unsafe {
+                                        std::mem::transmute(trampoline as *const c_void)
+                                    })
+                                },
+                                cb_kind: sys::$const_name,
+                            });
+                        }
+                    )*
+                }
+            ).into()
+        }
     }
 }
 
@@ -503,7 +622,14 @@ macro_rules! define_syscalls_callbacks {
     };
 }
 
-/// (Callback) Runs when proc_start_linux recieves the `AuxvValues` for a given process.
+/// (Callback) Runs when proc_start_linux recieves the [`AuxvValues`] for a given process.
+///
+/// Can be treated as a "on program start" callback, but one which provides a lot of
+/// info about the contents of the initial program state and how it is being loaded.
+/// The state at time of callback is before the C runtime is initialized, and before
+/// the entrypoint is jumped to.
+///
+/// See [`AuxvValues`] to get a better understanding of the values provided.
 ///
 /// ### Args
 ///
@@ -517,8 +643,8 @@ macro_rules! define_syscalls_callbacks {
 /// use panda::plugins::proc_start_linux::AuxvValues;
 ///
 /// #[panda::on_rec_auxv]
-/// fn callback(cpu: &mut CPUState, tb: &mut TranslationBlock, auxv: AuxvValues) {
-///     // do stuff
+/// fn on_proc_start(cpu: &mut CPUState, tb: &mut TranslationBlock, auxv: AuxvValues) {
+///     // do stuff when a process starts
 /// }
 /// ```
 ///
